using System;
using System.Collections.Generic;
using System.Data.Common;
using System.Data;

namespace Aitex.Core.RT.DBCore
{
    public interface IDataBase
    {
        string GetSqlByNameType(string name, Type type);

        int ExecuteNonQuery(string sql, bool isChangeDB = true);

        int ExecuteNonQuery(string sql, string[] columnsName, params object[] args);

        DbDataReader ExecuteReader(string sql);

        object ExecuteScalar(string sql, bool isChangeDB = true);

        DataSet ExecuteDataSet(string sql);

        DataTable ExecuteDataTable(string sql);

        bool ExcuteTransAction(List<string> sqlList);

        string GetLocalDictionary();
    }
}
